@@ -274,12 +274,18 @@ static void update_input(void)
 #endif
 }
 #else /* MAEMO */
+#include <gtk/gtk.h>
 static void update_input(void)
 {
 	extern int g_maemo_opts;
 	extern int maemo_x11_update_keys();
 	if (g_maemo_opts&8)
 		maemo_x11_update_keys();
+	else{
+		/* process GTK+ events */
+		while (gtk_events_pending())
+			gtk_main_iteration();
+	}
 }
 #endif
 
