@@ -31,22 +31,25 @@
                                 "',path='" CALL_SIGNAL_PATH \
                                 "',member='" CALL_INCOMING_SIG "'"
 
-osso_context_t* osso;
+osso_context_t* osso = NULL;
 int bRunning = TRUE;
 extern int bKeepDisplayOn;
 extern int bAutosaveOnExit;
+extern int cornerActions[4];
 extern char keys_config_file[MAXPATHLEN];
-static pthread_t display_thread = (pthread_t)-1;
+int bGameGripperFix = FALSE;
+static pthread_t display_thread = (pthread_t)0;
 int g_layer_x = (X_RES - D_WIDTH) / 2;
 int g_layer_y = (Y_RES - D_HEIGHT) / 2;
 int g_layer_w = D_WIDTH, g_layer_h = D_HEIGHT;
 
 static GdkImage *image;
 static HildonAnimationActor *actor;
-static GtkWidget *window, *drawing;
+static GtkWidget *window, *drawing = NULL;
 
 static int pl_buf_w, pl_buf_h;
 int keymap[65536];
+int direction_keys[4];
 
 // map psx4m compatible keymap to PSX keys
 static const unsigned char keymap2[14] = {
@@ -73,12 +76,123 @@ void hildon_quit()
 	exit(0);
 }
 
-static void
-window_key_proxy(GtkWidget *widget,
-		     GdkEventKey *event,
-		     gpointer user_data)
+gdouble press_x = -1;
+gdouble press_y = -1;
+
+int maemo_x11_update_keys();
+void show_notification(char* text);
+
+void change_slot(int delta)
 {
-	return key_press_event(event->hardware_keycode, event->type == GDK_KEY_PRESS?1:(event->type == GDK_KEY_RELEASE?2:0) );
+	state_slot += delta;
+	if (state_slot > 9)
+		state_slot = 0;
+	else if (state_slot < 0)
+		state_slot = 9;
+	char message[50];
+	sprintf(message,"Savestate slot: %i",state_slot + 1);
+	show_notification(message);
+}
+
+void save(int state_slot)
+{
+	emu_save_state(state_slot);
+	char buf[MAXPATHLEN];
+	if (image && image->mem){
+		sprintf (buf,"/opt/maemo/usr/games/screenshots%s.%3.3d",file_name,state_slot);
+		writepng(buf, image->mem, pl_buf_w,pl_buf_h);
+	}
+	char message[50];
+	sprintf(message,"Saved savestate slot: %i",state_slot + 1);
+	show_notification(message);
+}
+
+void quit()
+{
+	if (bAutosaveOnExit){
+		show_notification("Autosaving");
+		emu_save_state(99);
+		char buf[MAXPATHLEN];
+		if (image && image->mem){
+			sprintf (buf,"/opt/maemo/usr/games/screenshots%s.%3.3d",file_name,99);
+			writepng(buf, image->mem, pl_buf_w,pl_buf_h);
+		}
+	}
+	hildon_quit();
+}
+
+int show_confirmbox(char* text)
+{
+	if (!window)
+		return TRUE;
+
+	GtkWidget *dialog;
+	dialog = gtk_message_dialog_new (GTK_WINDOW(window),
+									 GTK_DIALOG_DESTROY_WITH_PARENT,
+									 GTK_MESSAGE_QUESTION,
+									 GTK_BUTTONS_YES_NO,
+									 text);
+	gint result = gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+	if (result == GTK_RESPONSE_YES)
+		return TRUE;
+	return FALSE;
+}
+
+static void
+window_button_proxy(GtkWidget *widget,
+  				    GdkEventButton *event,
+				    gpointer user_data)
+{
+	int corner = -1;
+	int sens = 100;
+
+	switch (event->type){
+	case GDK_BUTTON_PRESS:
+		//printf("GDK_BUTTON_PRESS: x=%f y=%f\n", event->x, event->y);
+		press_x = event->x;
+		press_y = event->y;
+		break;
+	case GDK_BUTTON_RELEASE:
+		//printf("GDK_BUTTON_RELEASE: x=%f y=%f\n", event->x, event->y);
+		if (press_x < sens && press_y < sens && event->x < sens && event->y < sens)
+			corner = 0;
+		else if (press_x > 800 - sens && press_y < sens && event->x > 800 - sens && event->y < sens)
+			corner = 1;
+		else if (press_x > 800 - sens && press_y > 480 - sens && event->x > 800 - sens && event->y > 480 - sens)
+			corner = 2;
+		else if (press_x < sens && press_y > 480 - sens && event->x < sens && event->y > 480 - sens)
+			corner = 3;
+
+		press_x = -1;
+		press_y = -1;
+		break;
+	default:
+		break;
+	}
+
+	if (corner >= 0){
+		switch (cornerActions[corner]){
+			case 1:
+				if (show_confirmbox("Save savestate?"))
+					save(state_slot);
+				break;
+			case 2:
+				if (show_confirmbox("Load savestate?"))
+					emu_load_state(state_slot);
+				break;
+			case 3:
+				change_slot(1);
+				break;
+			case 4:
+				change_slot(-1);
+				break;
+			case 5:
+				if (show_confirmbox("Quit?"))
+					quit();
+				break;
+		}
+	}
 }
 
 static void *displayThread(void *arg)
@@ -109,27 +223,33 @@ static void *displayThread(void *arg)
 
 void show_notification(char* text)
 {
-    GtkWidget* banner = hildon_banner_show_information(GTK_WIDGET(window), NULL, text);
-    hildon_banner_set_timeout(HILDON_BANNER(banner), 3000);
-	/*DBusConnection* session_bus = (DBusConnection*)osso_get_dbus_connection(osso);
-	DBusMessageIter args;
-	DBusMessage*msg = dbus_message_new_method_call("org.freedesktop.Notifications",
-												   "/org/freedesktop/Notifications",
-												   "org.freedesktop.Notifications",
-												   "SystemNoteInfoprint");
-	if (msg) {
-		dbus_message_iter_init_append(msg, &args);
-		char* param = text;
-		if (dbus_message_iter_append_basic(&args, DBUS_TYPE_STRING, &param)) {
-			dbus_connection_send(session_bus, msg, NULL);
-			dbus_connection_flush(session_bus);
+	if (window){
+		GtkWidget* banner = hildon_banner_show_information(GTK_WIDGET(window), NULL, text);
+		hildon_banner_set_timeout(HILDON_BANNER(banner), 3000);
+	}else{
+		DBusConnection* session_bus = (DBusConnection*)osso_get_dbus_connection(osso);
+		DBusMessageIter args;
+		DBusMessage*msg = dbus_message_new_method_call("org.freedesktop.Notifications",
+													   "/org/freedesktop/Notifications",
+													   "org.freedesktop.Notifications",
+													   "SystemNoteInfoprint");
+		if (msg) {
+			dbus_message_iter_init_append(msg, &args);
+			char* param = text;
+			if (dbus_message_iter_append_basic(&args, DBUS_TYPE_STRING, &param)) {
+				dbus_connection_send(session_bus, msg, NULL);
+				dbus_connection_flush(session_bus);
+			}
+			dbus_message_unref(msg);
 		}
-		dbus_message_unref(msg);
-	}*/
+	}
 }
 
 void show_messagebox(char* text)
 {
+	if (!window)
+		return;
+
 	GtkWidget *dialog;
 	dialog = gtk_message_dialog_new (GTK_WINDOW(window),
 									 GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -289,18 +409,71 @@ static DBusHandlerResult on_msg_recieved(DBusConnection* connection G_GNUC_UNUSE
 	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
 
+/*
+// Dirty workaround for the hardware problem (three keys pressed product four keys pressed for some keys combination like a+x+up arrow)
+guint gsource = 0;
+GdkEventKey key_event;
+
+static gboolean
+keys_handler(GtkWidget *widget)
+{
+	g_source_remove(gsource);
+	gsource = 0;
+	key_press_event(key_event.hardware_keycode, key_event.type == GDK_KEY_PRESS ? 1 : (key_event.type == GDK_KEY_RELEASE ? 2 : 0) );
+	return TRUE;
+}
+*/
+static void
+window_key_proxy(GtkWidget *widget,
+		     GdkEventKey *event,
+		     gpointer user_data)
+{
+	/*if (bGameGripperFix){
+		//printf("Key: %i %s time %i \n", event->hardware_keycode, event->type == GDK_KEY_PRESS ? "Pressed" : (event->type == GDK_KEY_RELEASE ? "Released" : "Unknown"), event->time);
+
+		//Trap the only case that happens with the game gripper:
+		if (event->hardware_keycode != 33 && event->hardware_keycode != 111 && event->hardware_keycode != 38 && event->hardware_keycode != 53){
+			return key_press_event(event->hardware_keycode, event->type == GDK_KEY_PRESS ? 1 : (event->type == GDK_KEY_RELEASE ? 2 : 0) );
+		}else{
+			if (gsource){
+				g_source_remove(gsource);
+				gsource = 0;
+				//Never replace a key release event (some buttons may remain pressed)
+				if (key_event.type == GDK_KEY_RELEASE || key_event.hardware_keycode != 33)
+					key_press_event(key_event.hardware_keycode, key_event.type == GDK_KEY_PRESS ? 1 : (key_event.type == GDK_KEY_RELEASE ? 2 : 0) );
+				else{
+					//printf("Replacing Key: %i %s time %i \n", key_event.hardware_keycode, key_event.type == GDK_KEY_PRESS ? "Pressed" : (key_event.type == GDK_KEY_RELEASE ? "Released" : "Unknown"), key_event.time);
+					//printf("         with: %i %s time %i \n", event->hardware_keycode, event->type == GDK_KEY_PRESS ? "Pressed" : (event->type == GDK_KEY_RELEASE ? "Released" : "Unknown"), event->time);
+				}
+			}
+
+			//Set a timer to elaborate the key event:
+			key_event = *event;
+			gsource = g_timeout_add(40, (GSourceFunc)keys_handler, (gpointer)window);
+		}
+	}else{*/
+		//printf("Key: %i %s time %i \n", event->hardware_keycode, event->type == GDK_KEY_PRESS ? "Pressed" : (event->type == GDK_KEY_RELEASE ? "Released" : "Unknown"), event->time);
+		return key_press_event(event->hardware_keycode, event->type == GDK_KEY_PRESS ? 1 : (event->type == GDK_KEY_RELEASE ? 2 : 0) );
+	//}
+}
+
+int last_key_pressed = 0;
 inline void key_press_event(int key2,int type)
 {
-	int  psxkey1 = -1, psxkey2 = -1;
+	int psxkey1 = -1, psxkey2 = -1;
 	int key=keymap[key2];
 
 	if (key < 0)
 		return;
+	
+	if (type == 1 && key2 == last_key_pressed)
+		return;
+	last_key_pressed = type == 1 ? key2 : 0;
 
 	//printf("Key: %i %s\n", key2, type == 1 ? "Pressed" : (type == 2 ? "Released" : "Unknown"));
 	if (key < ARRAY_SIZE(keymap2)){
 		psxkey1 = keymap2[key];
-		/*switch(keymap2[key]){
+		/*switch(psxkey1){
 			case DKEY_LEFT:
 				printf("  DKEY_LEFT\n");
 				break;
@@ -328,16 +501,7 @@ inline void key_press_event(int key2,int type)
 		}*/
 	}else switch (key) {
 		case 14:
-			if (bAutosaveOnExit){
-			    show_notification("Autosaving");
-				emu_save_state(99);
-				char buf[MAXPATHLEN];
-				if (image && image->mem){
-                    sprintf (buf,"/opt/maemo/usr/games/screenshots%s.%3.3d",file_name,99);
-                    writepng(buf, image->mem, pl_buf_w,pl_buf_h);
-				}
-			}
-			hildon_quit();
+			quit();
 			break;
 		case 15:
 			psxkey1 = DKEY_UP;
@@ -357,37 +521,19 @@ inline void key_press_event(int key2,int type)
 			break;
 		case 19:
 			if (type == 1)
-			{
-				emu_save_state(state_slot);
-				char buf[MAXPATHLEN];
-				if (image && image->mem){
-                    sprintf (buf,"/opt/maemo/usr/games/screenshots%s.%3.3d",file_name,state_slot);
-                    writepng(buf, image->mem, pl_buf_w,pl_buf_h);
-				}
-				char message[50];
-				sprintf(message,"Saved savestate slot: %i",state_slot + 1);
-				show_notification(message);
-			}
+				save(state_slot);
 			return;
 		case 20:
 			if (type == 1)
 				emu_load_state(state_slot);
 			return;
 		case 21:
-			if (type == 1){
-				state_slot=(state_slot<9)?state_slot+1:0;
-				char message[50];
-				sprintf(message,"Savestate slot: %i",state_slot + 1);
-				show_notification(message);
-			}
+			if (type == 1)
+				change_slot(1);
 			return;
 		case 22:
-			if (type == 1){
-				state_slot=(state_slot>0)?state_slot-1:9;
-				char message[50];
-				sprintf (message,"Savestate slot: %i",state_slot + 1);
-				show_notification(message);
-			}
+			if (type == 1)
+				change_slot(-1);
 			return;
 		case 23:
 			if (type == 1){
@@ -426,6 +572,7 @@ void set_accel_multipliers()
 	accelOptions.yMultiplier = 255.0 / ( (accelOptions.maxValue - accelOptions.sens) * 2.0);
 }
 
+#include <gdk/gdkx.h>
 int maemo_init(int *argc, char ***argv)
 {
 	osso = osso_initialize("pcsxrearmed", PACKAGE_VERSION, FALSE, NULL);
@@ -445,21 +592,38 @@ int maemo_init(int *argc, char ***argv)
 	int ch;
 	int i=0;
 	for (i=0;i<65536;i++)
-		keymap[i]=164;
+		keymap[i]=-1;
 	if (NULL != pFile) {
 		for(i=0;i<24;i++){
 			fscanf(pFile, "%i",&ch);
 			keymap[ch]=i;
+			if (i < 4)
+				direction_keys[i] = ch;
 		}
 		fclose(pFile);
 	}
 
+	//Check if the key mappings are for the Game Gripper to activate the "dirty workaround" :p
+	if (direction_keys[0] == 38 && direction_keys[1] == 40 &&
+		direction_keys[2] == 25 && direction_keys[3] == 53){
+		bGameGripperFix = TRUE;
+		printf("Game Gripper detected\n");
+	}
+
 	if (!(g_maemo_opts&8)){
 		gtk_init (argc, argv);
 
 		window = hildon_stackable_window_new ();
 		gtk_widget_realize (window);
 		gtk_window_fullscreen (GTK_WINDOW(window));
+
+		if (cornerActions[0] + cornerActions[1] + cornerActions[2] + cornerActions[3] > 0){
+			g_signal_connect (G_OBJECT (window), "button_release_event",
+						G_CALLBACK (window_button_proxy), NULL);
+			g_signal_connect (G_OBJECT (window), "button_press_event",
+						G_CALLBACK (window_button_proxy), NULL);
+		}
+
 		g_signal_connect (G_OBJECT (window), "key-press-event",
 					G_CALLBACK (window_key_proxy), NULL);
 		g_signal_connect (G_OBJECT (window), "key-release-event",
@@ -482,6 +646,12 @@ int maemo_init(int *argc, char ***argv)
 
 		gtk_widget_show_all (GTK_WIDGET (actor));
 		gtk_widget_show_all (GTK_WIDGET (window));
+	}else{
+		gtk_init (argc, argv);
+		/*GdkScreen* scr = gdk_screen_get_default();
+		window = GTK_WIDGET(gdk_screen_get_root_window(scr));
+		if (!window)
+			window = GTK_WIDGET(gdk_get_default_root_window());*/
 	}
 
 	set_accel_multipliers();
@@ -505,6 +675,7 @@ void maemo_finish()
 		osso_deinitialize(osso);
 		osso = NULL;
 	}
+	printf("Exiting\n");
 }
 
 void menu_loop(void)
@@ -593,11 +764,11 @@ void *hildon_flip(void)
 
 	if (g_maemo_opts & 8)
 		maemo_x11_update_keys();
-	else
+	else{
 		/* process GTK+ events */
 		while (gtk_events_pending())
 			gtk_main_iteration();
-
+	}
 	return pl_vout_buf;
 }
 
