@@ -32,6 +32,7 @@ extern int iUseInterpolation;
 accel_option accelOptions;
 int g_opts = 0;
 int g_maemo_opts;
+int cornerActions[4] = {0,0,0,0};
 int bKeepDisplayOn = FALSE;
 int bAutosaveOnExit = FALSE;
 char file_name[MAXPATHLEN];
@@ -104,8 +105,18 @@ void PrintHelp()
     printf("                   (Default disabled)\n");
 	printf("  -keys FILE     : File with keys configuration\n");
     printf("                   (Default /opt/psx4m/keys)\n");
-
-
+	printf("  -corners VALUE : Define actions for click on the\n");
+    printf("                   display corners\n");
+    printf("                   VALUE is a four digit number, each number\n");
+    printf("                   represent a corner (topleft, topright,\n");
+    printf("                   bottomright and bottomleft\n");
+    printf("                   Actions:\n");
+    printf("                   0=No action\n");
+    printf("                   1=Save\n");
+    printf("                   2=Load\n");
+    printf("                   3=Change slot (+1)\n");
+    printf("                   4=Change slot (-1)\n");
+    printf("                   5=Quit\n");
 
 	printf("\nGPU Options:\n");
 	printf("  -gles          : Use the GLES plugin (gpuGLES.so)\n");
@@ -262,6 +273,15 @@ int maemo_main(int argc, char **argv)
 		else if (!strcmp(argv[i], "-keys")) 				sprintf(keys_config_file, "%s", argv[++i]);
 		else if (!strcmp(argv[i], "-autosave")) 		    bAutosaveOnExit = TRUE;
 		else if (!strcmp(argv[i], "-disc")) 		        discNumber = atol(argv[++i]);
+		else if (!strcmp(argv[i], "-corners")){
+			int j = 0;
+			i++;
+			char num[2];
+			for (j=0; j<strlen(argv[i]); j++){
+				strncpy(num, argv[i] + j, 1);
+				cornerActions[j] = atoi(num);
+			}
+		}
 
 		else if (!strcmp(argv[i], "-spu_reverb"))           { if (atol(argv[++i]) > 0) iUseReverb = 2; }
 		else if (!strcmp(argv[i], "-spu_interpolation")) 	iUseInterpolation = atol(argv[++i]);
