@@ -39,11 +39,12 @@
 #include "gpuTexture.h"
 #include "gpuFps.h"
 #include "gpuPrim.h"
-
+#include "../../frontend/plugin_lib.h"
 //#include "NoPic.h"
 
 #include "gpuStdafx.h"
 
+static const struct rearmed_cbs *icbs;
 short g_m1=255,g_m2=255,g_m3=255;
 short DrawSemiTrans=FALSE;
 short Ymin;
@@ -702,6 +703,7 @@ else                                                  // no skip ?
   if(iDrawnSomething)  flipEGL();
  }
 
+icbs->pl_vout_flip(); // For the accel in hildon_flip()
 iDrawnSomething=0;
 
 //----------------------------------------------------//
@@ -2652,7 +2654,6 @@ void CALLBACK GPUdisplayFlags(unsigned long dwFlags)
 }
 
 // pcsx-rearmed callbacks
-#include "../../frontend/plugin_lib.h"
 void CALLBACK GPUrearmedCallbacks(const void **cbs)
 {
     rearmed_get_layer_pos = cbs[0];
@@ -2672,6 +2673,8 @@ void CALLBACK GPUrearmedCallbacks(const void **cbs)
 
 	//bUseFrameSkip = cbs_->frameskip ? TRUE : FALSE; 
 	bUseFrameSkip = FALSE; 
+
+	icbs = cbs_;
 }
 
 static void flipEGL(void)
